
import os, sys
import pygtk

if sys.platform == 'win32':
        os.environ['PATH'] += ";lib;"

import gtk
import gobject
assert gtk.pygtk_version >= (2,4,0), "pygtk should be >= 2.4.0"
import gtk.glade
import traceback

try:
    import gnome
    import gnome.ui
    import gconf
except:
    gnome = None

import DSConnection
import NodeProperties
from NewUserAccount import NewUserAccountDialog
from NewGroup import NewGroupDialog
from Globals import *

import traceback

class QuestionDialog:
    def __init__(self, title, message):
        gxml = gtk.glade.XML(GLADE_FILE, "questionDialog")
        self.dlg = gxml.get_widget("questionDialog")
        messageLabel = gxml.get_widget("messageLabel")
        messageTitle = gxml.get_widget("messageTitle")

        messageLabel.set_text(message)
        messageLabel.set_use_markup(True)

        messageTitle.set_text('<b>%s</b>' % (title))
        messageTitle.set_use_markup(True)

        self.dlg.set_title(title)

    def Run(self):
        res = self.dlg.run()
        self.dlg.destroy()
        return (res == gtk.RESPONSE_OK)

class ErrorDialog:
    def __init__(self, title, message):
        gxml = gtk.glade.XML(GLADE_FILE, "errorDialog")
        self.dlg = gxml.get_widget("errorDialog")
        messageLabel = gxml.get_widget("errorMessage")
        messageTitle = gxml.get_widget("errorTitle")

        messageLabel.set_text(message)
        messageLabel.set_use_markup(True)

        messageTitle.set_text('<b>%s</b>' % (title))
        messageTitle.set_use_markup(True)

        self.dlg.set_title(title)

    def Run(self):
        res = self.dlg.run()
        self.dlg.destroy()
        return (res == gtk.RESPONSE_OK)

class FolderNameDialog:
    def __init__(self):
        gxml = gtk.glade.XML(GLADE_FILE, "FolderName")        
        self.dlg = gxml.get_widget("FolderName")
        self.foldername = gxml.get_widget("foldername")

    def Run(self):
        res = self.dlg.run()
        try:
            if (res == gtk.RESPONSE_OK):
                return self.foldername.get_text()
        finally:
            self.dlg.destroy()

class Main:

    TARGETS = [
            ('EDSADMIN_NODE_OBJECT', gtk.TARGET_SAME_APP, 0)
    ]

    def __init__(self):

        if (gnome):
            gtk.window_set_default_icon_from_file(APP_ICON)
            program = gnome.init("DS Admin", "1.0")
            self.gconf_client = gconf.client_get_default()

        self.gxml = gtk.glade.XML(GLADE_FILE, "DS1 Admin")

        # a handle to pass data via drag and drop
        self.drag_drop_obj = None

        self.container_tree = self.gxml.get_widget("ContainerTree")
        self.contents_list = self.gxml.get_widget("ContentsList")
        self.containerDescription = self.gxml.get_widget("containerDescription")

        self.selected_folder = None

        # Connect the glade signals
        function_dict = dict([(f, getattr(self, f)) for f in dir(self)]) 
        self.gxml.signal_autoconnect(function_dict)
        self.InitGui()

        gtk.idle_add(self.InitConnection)
        gtk.main()

    def InitGui(self):
        self.InitContainerTreeView()
        self.InitContentsList()

    def InitConnection(self):

        try:
            self.DSConn = DSConnection.DSConnection.TryAutoConnect(self.ShowKerberosPasswordDialog)
            if not self.DSConn:
                raise Exception, "autoconnect failed"
            self.InitModels()
            return
        except Exception, err:
            print "Error: %s" % (err)
            traceback.print_exc(file=sys.stdout)
            self.ShowLoginDialog()

    def ShowKerberosPasswordDialog(self, principal=None):
        gxml = gtk.glade.XML(GLADE_FILE, 'KerberosPassword')
        passwordDialog = gxml.get_widget('KerberosPassword')

        res = passwordDialog.run()

        if res == gtk.RESPONSE_CANCEL:
            return None

        else:
            password = gxml.get_widget('password').get_text()
            passwordDialog.destroy()
            return password
       
    def ShowLoginDialog(self):
        gxml = gtk.glade.XML(GLADE_FILE, 'loginDialog')
        loginDialog = gxml.get_widget('loginDialog')

        errorTemplate = '<span color="red"><b>%s</b></span>'
        errorLabel = gxml.get_widget('errorMessage')

        errorMessage = None
        loginComplete = False

        port = None

        # Detect ldap servers and prefill the hostname
        detectedHosts = DSConnection.DSConnection.LocateHosts()
        if (detectedHosts):

            model = gtk.ListStore(str, int)
            for host in detectedHosts:
                model.append(host)
            gxml.get_widget('hostname').set_model(model)
            gxml.get_widget('hostname').set_text_column(0)
            
            gxml.get_widget('hostname').child.set_text(detectedHosts[0][0])
            gxml.get_widget('port').set_text(str(detectedHosts[0][1]))
            port = detectedHosts[0][1]
            gxml.get_widget('username').grab_focus()

        while not (loginComplete):
            if (errorMessage):
                errorLabel.show()
                errorLabel.set_text(errorTemplate % (errorMessage))
                errorLabel.set_use_markup(True)
            else:
                errorLabel.hide()

            res = loginDialog.run()
            loginDialog.hide()

            if (res == gtk.RESPONSE_CANCEL):
                break
            
            hostname = gxml.get_widget('hostname').child.get_text()
            username = gxml.get_widget('username').get_text()
            password = gxml.get_widget('password').get_text()
            
            if (gxml.get_widget('advancedExpander').get_expanded()):
                try:
                    port = int(gxml.get_widget('port').get_text())
                except ValueError:
                    errorMessage = errorTemplate % ("Invalid port")
                    gxml.get_widget('port').grab_focus()
                    continue

                if (gxml.get_widget('crypt_auto').get_active()):
                    security = DSConnection.SECURITY_ANY
                elif (gxml.get_widget('crypt_tls').get_active()):
                    security = DSConnection.SECURITY_TLS
                elif (gxml.get_widget('crypt_ssl').get_active()):
                    security = DSConnection.SECURITY_SSL
                else:
                    security = DSConnection.SECURITY_NONE

                basedn = gxml.get_widget('basedn').get_text()

                self.DSConn = DSConnection.DSConnection( \
                        hostname, port, basedn, security)
            else:
                self.DSConn = DSConnection.DSConnection( \
                    hostname, port)
            try:
                self.DSConn.connect(username, password)
            except DSConnection.ConnectError, err:
                errorMessage = err
                gxml.get_widget('hostname').grab_focus()
                continue
            except DSConnection.AuthError, err:
                errorMessage = err
                gxml.get_widget('username').grab_focus()
                continue
            except Exception, err:
                errorMessage = err
                traceback.print_exc()
            else:
                loginComplete = True
        
        if (loginComplete):
            print "SecurityMode: %d" % (self.DSConn.security)
            self.InitModels()
            
    def InitModels(self):
        self.DSConn.RegisterHandler(DSConnection.Disconnected, self.OnServerDisconnected)
        self.containerModels = {}
        self.contents_list.set_model(None)
        self.InitContainerTreeModel()

    def OnServerDisconnected(self, error):
        NodeProperties.closeAllNodeDialogs()
        self.container_tree.set_model(None)
        self.contents_list.set_model(None)
        self.ShowLoginDialog()
    
    def InitContainerTreeModel(self):
        self.root = self.DSConn.getRootContainer()

        self.containers = {}
        containerTreeStore = gtk.TreeStore(gtk.gdk.Pixbuf, str, object)

        # Add "All Users" and "All Groups" items
        cont = self.DSConn.UsersVFolder()
        containerTreeStore.append(None, [self.folderpb, cont.Name(), cont])
        cont = self.DSConn.GroupsVFolder()
        containerTreeStore.append(None, [self.folderpb, cont.Name(), cont])
        self.PopulateContainerTreeStore(containerTreeStore, self.root)

        self.container_tree.set_model(containerTreeStore)
        root_iter = containerTreeStore.get_iter_root()
        
        self.container_tree.expand_row(containerTreeStore.get_path(root_iter), \
            False)

    def InitContainerTreeView(self):
        self.folderpb = images['folder_closed']
        self.openfolderpb = images['folder_open']

        col1 = gtk.TreeViewColumn("Name")
        
        cellpb = gtk.CellRendererPixbuf()
        cellpb.set_property("xalign", 0.0)
        col1.pack_start(cellpb, False)
        col1.add_attribute(cellpb, "pixbuf", 0)
        
        celltext = gtk.CellRendererText()
        celltext.set_property("xalign", 0.0)
        col1.pack_start(celltext, False)
        col1.add_attribute(celltext, "text", 1)
        
        self.container_tree.append_column(col1)
        self.container_tree.connect("row-activated", \
            self.OnContainerTreeRowActivated, None)

        self.container_tree.enable_model_drag_dest(self.TARGETS,
                                             gtk.gdk.ACTION_DEFAULT)
        self.container_tree.connect("drag_data_received",
                                 self.OnContainerTreeDragDataReceived)

    def InitContentsList(self):

        self.containerModels = {}

        self.userpb = images['stock_contact']
        self.grouppb = images['stock_contact_list']
        self.computerpb = images['stock_directory_server']
        self.userlistpb = images['stock_addressbook']

        col1 = gtk.TreeViewColumn("Display Name")
        cellpb = gtk.CellRendererPixbuf()
        cellpb.set_property("xalign", 0.0)
        col1.pack_start(cellpb, False)
        col1.add_attribute(cellpb, "pixbuf", 0)
        
        celltext = gtk.CellRendererText()
        celltext.set_property("xalign", 0.0)
        col1.pack_start(celltext, False)
        col1.set_cell_data_func(celltext, lambda column,cell,model,iter,*args: cell.set_property("text", model.get_value(iter, 2).Name()))
        col1.set_sort_column_id(1)

        self.contents_list.append_column(col1)

        col2 = gtk.TreeViewColumn("ID")
        celltext = gtk.CellRendererText()
        celltext.set_property("xalign", 0.0)
        col2.pack_start(celltext, False)
        col2.set_cell_data_func(celltext, lambda columns,cell,model,iter,*args: cell.set_property("text", model.get_value(iter, 2).PrettyPath().split('/')[-1]))
        #col2.set_sort_column_id(2)
        self.contents_list.append_column(col2)

        col3 = gtk.TreeViewColumn("Type")
        celltext = gtk.CellRendererText()
        celltext.set_property("xalign", 0.0)
        col3.pack_start(celltext, False)
        col3.set_cell_data_func(celltext, self.ContentsList_Col3Data)
        #col3.set_sort_column_id(3)
        self.contents_list.append_column(col3)

        self.contents_list.connect("row-activated", \
            self.OnContentsListRowActivated, None)

        self.contents_list.set_search_column(0)


        self.contents_list.enable_model_drag_source( gtk.gdk.BUTTON1_MASK,
                                                  self.TARGETS,
                                                  gtk.gdk.ACTION_DEFAULT|
                                                  gtk.gdk.ACTION_MOVE)

        self.contents_list.connect("drag_data_get", self.OnContentsListDragDataGet)

    def ContentsList_Col3Data(self, columns, cell, model, iter, *args):
        """Cell data function for the 'type' column of the contents list.

        Matches an implementation name to a "friendly" display name.
        """
        node = model.get_value(iter, 2)
        typeMap = {'IDSComputer':'Computer',
                    'IDSUser':'User',
                    'IDSGroup':'Group',
                    'IDSUserList':'User List'}
        
        nodetype = 'unknown'
        for impl in node.Implements():
            nodetype = typeMap.get(impl, nodetype)

        cell.set_property("text", nodetype)

    def PopulateContainerTreeStore(self, store, dscont, parent=None):
        iter = store.append(parent, [self.folderpb, dscont.Name(), dscont])
        # Store a handle to the location of the item in the model
        # for purposes of refreshing later
        self.containers[dscont.Path()] = (dscont, iter)

        children = dscont.getChildContainers()
        for child in children:
            self.PopulateContainerTreeStore(store, child, iter)
        
    def OnContainerTreeRowActivated(self, treeview, path, column, user_data):
        """Set the folder icon and show the children of the selected node"""

        previous = self.selected_folder
        if (previous):
            treeview.get_model().set_value(previous, 0, self.folderpb)
        iter = treeview.get_model().get_iter(path)
        container = treeview.get_model().get_value(iter, 2)
        treeview.get_model().set_value(iter, 0, self.openfolderpb)
        self.selected_folder = iter
        self.ShowContainerContents(container)

    def OnConnectActivate(self, menuitem, *args):
        self.ShowLoginDialog()

    def OnPropertiesClicked(self, toolbutton):
        selection = self.container_tree.get_selection()
        if not (selection):
            return
        model, iter = selection.get_selected()
        container = model.get_value(iter, 2)
        selection = self.contents_list.get_selection()
        model, iter = selection.get_selected()
        node = model.get_value(iter, 2)
        self.ShowNode(node)

    def OnContentsListRowActivated(self, treeview, path, column, user_data):
        iter = treeview.get_model().get_iter(path)
        node = treeview.get_model().get_value(iter, 2)
        self.ShowNode(node)


    def OnContainerTreeDragDataReceived(self, treeview, context, x, y, selection,
                                        info, etime):
        print selection.data
        if (self.drag_drop_obj and self.drag_drop_obj.Path() == selection.data):
            node = self.drag_drop_obj
            self.drag_drop_obj = None
            print node
            print "INFO: Drag and drop is not yet implemented"

    def OnContentsListDragDataGet(self, treeview, context, selection, target_id,
                                  etime):
        model, iter = self.contents_list.get_selection().get_selected()
        node = model.get_value(iter, 2)
        self.drag_drop_obj = node
        selection.set('EDSADMIN_NODE_OBJECT', 8, node.Path())

    def ShowNode(self, node):
        NodeProperties.showNodePropertiesDialog(node)
        
    def ShowContainerContents(self, container):

        if (container.description):
            self.containerDescription.set_text(container.description)
            self.containerDescription.show()
        else:
            self.containerDescription.hide()

        if self.containerModels.has_key(container.Path()):
            print "%s is cached" % (container.Path())
            if not (container.Path().endswith('type=search')):
                self.contents_list.set_model(self.containerModels[container.Path()])
                return
        
        model = gtk.ListStore(gtk.gdk.Pixbuf, str, object)
        children = container.getChildNodes()

        for child in children:
            name = child.Name()
            if (child.Implements('IDSComputer')):
                pb = self.computerpb
            elif (child.Implements('IDSUser')):
                pb = self.userpb
            elif (child.Implements('IDSGroup')):
                pb = self.grouppb
            elif (child.Implements('IDSUserList')):
                pb = self.userlistpb
            else:
                pb = None

            model.append((pb, name, child))

        self.containerModels[container.Path()] = model
        
        self.contents_list.set_model(model)

    def ShowContentsListContextMenu(self, button, activate_time, node=None):

        menu = gtk.Menu()
        menuItem = gtk.MenuItem('New _user account', True)
        menuItem.connect('activate', self.OnNewUserAccountActivated)
        menu.append(menuItem)
        menuItem.show()

        menuItem = gtk.MenuItem('New _group', True)
        menuItem.connect('activate', self.OnNewGroupActivated)
        menu.append(menuItem)
        menuItem.show()

        if (node):
            if (node.Implements('IDSUser')):
                separator = gtk.SeparatorMenuItem()
                menu.append(separator)
                separator.show()
                menuItem = gtk.MenuItem('Set _password...')
                menuItem.connect('activate', self.OnSetPasswordActivated)
                menu.append(menuItem)
                menuItem.show()
            
        
            separator = gtk.SeparatorMenuItem()
            menu.append(separator)
            separator.show()
            
            menuItem = gtk.MenuItem('_Delete...')
            menuItem.connect('activate', self.OnDeleteActivated)
            menu.append(menuItem)
            menuItem.show()
                

        
        menu.popup(None, None, None, button, activate_time)

    def OnContentsListButtonPressEvent(self, obj, event, *args):
        if (event.button == 3):

            # The node to be passed on for creation of node specific items
            # such as "Set Password"
            node = None

            # Check whether a list item was under the mouse pointer
            res = self.contents_list.get_path_at_pos(
                    event.x, event.y
                )

            if (res):
                # yup, there was. Select it.
                path = res[0]
                selection = self.contents_list.get_selection()
                selection.select_path(path)

                listModel = self.contents_list.get_model()
                iter = listModel.get_iter(path)
                node = listModel.get_value(iter, 2)
                

            print type(event.time), event.time
            self.ShowContentsListContextMenu(event.button, event.time, node)
            return True

        return False

    def OnContentsListPopupMenu(self, obj, *args):
        selection = self.contents_list.get_selection()
        listModel, iter = selection.get_selected()
        if (iter):
            node = listModel.get_value(iter, 2)
            eventButton = 3
            self.ShowContentsListContextMenu(eventButton, gtk.get_current_event_time(), node)
            return True

        return False

    def ShowContainerTreeContextMenu(self, button, activate_time, treeIter=None):
        node = None
        if (treeIter):
            treeModel = self.container_tree.get_model()
            node = treeModel.get_value(treeIter, 2)

        if (node) and (not node.Path().endswith('type=search')):
            menu = gtk.Menu()
            menuItem = gtk.MenuItem('Create folder', True)
            menuItem.connect('activate', self.OnNewFolderActivated, treeIter)
            menu.append(menuItem)
            menuItem.show()

            # For containers other than the root node, allow deletion
            if (node != self.root):
                    
                menuItem = gtk.MenuItem('Delete', True)
                menuItem.connect('activate', self.OnDeleteFolderActivated, treeIter)
                menu.append(menuItem)
                menuItem.show()

            menu.popup(None, None, None, button, activate_time)


    def OnContainerTreeButtonPressEvent(self, obj, event, *args):
        if (event.button == 3):

            # The node to be passed on for creation of node specific items
            node = None

            # Check whether a list item was under the mouse pointer
            res = self.container_tree.get_path_at_pos(
                    event.x, event.y
                )

            if (res):
                # yup, there was. Select it.
                path = res[0]
                selection = self.container_tree.get_selection()
                selection.select_path(path)

                treeModel = self.container_tree.get_model()
                treeIter = treeModel.get_iter(path)
                #node = treeModel.get_value(iter, 2)
                

            
            self.ShowContainerTreeContextMenu(event.button, event.time, treeIter)
            return True

        return False

    def OnContainerTreePopupMenu(self, obj, *args):
        selection = self.container_tree.get_selection()
        treeModel, treeIter = selection.get_selected()
        if (treeIter):
            eventButton = 3
            self.ShowContainerTreeContextMenu(eventButton, gtk.get_current_event_time(), treeIter)
            return True

        return False

    def OnNewFolderActivated(self, menuitem, treeIter, *args):
    
        name = FolderNameDialog().Run()
        
        if not (name):
            # The operation was cancelled
            return

        print "Creating folder named: %s" % name
        treeModel = self.container_tree.get_model()
        node = treeModel.get_value(treeIter, 2)
        node.CreateFolder(name)
        for child in node.getChildContainers():
            if not self.containers.has_key(child.Path()):
                childIter = treeModel.append(treeIter, [self.folderpb, child.Name(), child])
                self.containers[child.Path()] = (child, childIter)


    def OnDeleteFolderActivated(self, menuitem, treeIter, *args):
        treeModel = self.container_tree.get_model()
        node = treeModel.get_value(treeIter, 2)
        
        title = "Delete folder?"    
        message = "Are you sure you want to completely delete the folder: %s?" % (node.Name())

        confirmation = QuestionDialog(title, message).Run()
        if not (confirmation):
            return        
        
        nodePath = node.Path()
        try:
            node.Delete()
            treeModel = self.container_tree.get_model()
            if (treeModel.is_ancestor(treeIter, self.selected_folder) or \
                    treeModel.get_string_from_iter(treeIter) == \
                    treeModel.get_string_from_iter(self.selected_folder)):
                print "setting selected_folder = None"
                self.selected_folder = None
                self.contents_list.set_model(None)
            # Delete the row from the tree
            treeModel.remove(treeIter)
            
            # remove this node and its children from the cache            
            for path in self.containerModels.keys():
                if path.endswith(nodePath):
                    del self.containerModels[path]

            del self.containers[nodePath]
        except Exception, err:
            dlg = ErrorDialog('Error while deleting', str(err))
            dlg.Run()
            
    
    def OnNewUserAccountClicked(self, toolbutton, *args):
        self.NewUserAccount()

    def OnNewUserAccountActivated(self, menuitem, *args):
        self.NewUserAccount()

    def NewUserAccount(self):
        selection = self.container_tree.get_selection()
        model, treeIter = selection.get_selected()
        container = model.get_value(treeIter, 2)
        newUserDialog = NewUserAccountDialog(container)
        newUserDialog.connect('account-created', self.OnContainerChanged)

    def OnDeleteClicked(self, toolbutton, *args):
        self.Delete()

    def OnDeleteActivated(self, menuitem, *args):
        self.Delete()

    def Delete(self):
        selection = self.container_tree.get_selection()
        model, iter = selection.get_selected()
        container = model.get_value(iter, 2)
        selection = self.contents_list.get_selection()
        model, iter = selection.get_selected()
        node = model.get_value(iter, 2)
        parent = node.parent

        title = "Delete %s?"
        message = "Are you sure you want to completely delete the %s: %s?"
        msgMap = {'IDSUser': 'user account',
                  'IDSGroup': 'group',
                  'IDSComputer': 'computer account',
                  'IDSUserList': 'user list'}

        for impl in node.Implements():
            msg = msgMap.get(impl, 'object')
        
        title = title % msg
        message = message % (msg, node.Name())

        confirmation = QuestionDialog(title, message).Run()
        if not (confirmation):
            return
        
        try:
            NodeProperties.closeNodePropertiesDialog(node)
            container.DeleteNode(node)
            self.OnContainerChanged(container=container)
            if (parent != container):
                self.containers[parent.Path()][0].Refresh()
                self.OnContainerChanged(container=parent)
        except Exception, err:
            dlg = ErrorDialog('Error while deleting', str(err))
            dlg.Run()

    def OnSetPasswordActivated(self, menuitem, *args):
        selection = self.contents_list.get_selection()
        model, iter = selection.get_selected()
        node = model.get_value(iter, 2)
        
        dlg = NodeProperties.PasswordChangeDialog(node)
        dlg.run()

    def OnContainerChanged(self, obj=None, container=None):
        if self.containerModels.has_key(container.Path()):
            del self.containerModels[container.Path()]

        selection = self.container_tree.get_selection()
        model, iter = selection.get_selected()
        selected_container = model.get_value(iter, 2)

        if selected_container.Path() == container.Path():
            self.ShowContainerContents(container)

    def OnNewGroupClicked(self, toolbutton, *args):
        self.NewGroup()

    def OnNewGroupActivated(self, menuitem, *args):
        self.NewGroup()

    def OnAboutActivated(self, menuitem, *args):
        gxml = gtk.glade.XML(GLADE_FILE, "aboutdialog1")
        dlg = gxml.get_widget("aboutdialog1")        
        dlg.run()

    def NewGroup(self):
        selection = self.container_tree.get_selection()
        model, iter = selection.get_selected()
        container = model.get_value(iter, 2)
        newGroupDialog = NewGroupDialog(container)
        newGroupDialog.connect('account-created', self.OnContainerChanged)

    def Quit(self, *args):
        gtk.main_quit()

    def OnWindowDeleteEvent(self, obj, *args):
        self.Quit()

        
if __name__ == '__main__':
    Main()
